import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'package:connectivity/connectivity.dart';
import 'package:dio/dio.dart';
import 'package:http/http.dart';
import 'package:senior_authentication/src/models/refresh_token_model.dart';
import 'package:senior_authentication/src/models/user.dart';
import 'package:senior_authentication/src/utils/utils.dart';

import './models/models.dart';
import './utils/exceptions.dart';
import './utils/preference_storage.dart';

// ignore_for_file: avoid_dynamic_calls

AuthenticationRepository platformData = AuthenticationRepository();

/// Indicates the possible authentication status of senior X Platform.
enum AuthenticationStatus {
  /// When a related request fails or any other exception
  unknown,

  /// When phone isn't connected on network
  offline,

  /// Status when user is authenticated on Senior X Platform.
  authenticated,

  /// Status when user is not authenticated on Senior X Platform.
  unauthenticated,
}

///
class AuthenticationRepository {
  static AuthenticationRepository? _instance;

  /// {@macro authentication_repository}
  ///
  /// If no [urlBase] is provided, production enpoint will be used.
  factory AuthenticationRepository(
      {String urlBase =
          'https://platform.senior.com.br/t/senior.com.br/bridge/1.0/rest'}) {
    _instance ??= AuthenticationRepository._(urlBase);
    return _instance!;
  }

  AuthenticationRepository._(this.urlBase);

  /// senior X Platform communication URL.
  String urlBase;

  final Map<String, String> _headers = {
    'Content-Type': 'application/json',
  };

  final _controller = StreamController<AuthenticationStatus>();

  final Dio _dio = Dio();

  Dio get dio {
    return PlatformInterceptor().addInterceptor(urlBase, _dio);
  }

  /// Gets the actual authentication status.
  ///
  /// View [AuthenticationStatus] for details.
  Stream<AuthenticationStatus> get status async* {
    yield AuthenticationStatus.unknown;
    yield* _controller.stream;
  }

  /// Returns the current cached token.
  /// Defaults to [Token.empty] if there is no cached token.
  Future<Token> get currentToken async {
    return await Storage.getToken();
  }

  /// Returns the current logged user
  Future<User?> get loggedUser async {
    return await Storage.getLoggedUser();
  }

  /// Check authorization status
  void initialize() async {
    var token = await Storage.getToken();

    var connectivityResult = await (Connectivity().checkConnectivity());
    if (connectivityResult == ConnectivityResult.none) {
      _controller.add(AuthenticationStatus.offline);
    }

    if (token.isEmpty) _controller.add(AuthenticationStatus.unauthenticated);

    try {
      await getUser(token.access_token,
          autoRefreshToken: true, refreshToken: token.refresh_token);
      _controller.add(AuthenticationStatus.authenticated);
    } on RefreshTokenUnauthorized {
      _controller.add(AuthenticationStatus.unauthenticated);
    } on RefreshTokenFailure {
      _controller.add(AuthenticationStatus.unknown);
    } on Exception {
      _controller.add(AuthenticationStatus.unknown);
    }
  }

  /// Get info from user requestings /getUser on platform
  Future<User?> getUser(String accessToken,
      {bool autoRefreshToken = false, String? refreshToken}) async {
    try {
      var response = await dio.get(
        '$urlBase/platform/user/queries/getUser',
        options: Options(
          headers: {'Accept': 'application/json;seniorx.version=8'},
        ),
        queryParameters: {'includePhoto': true},
      );

      if (response.statusCode == 200) {
        var data = response.data;
        var user = data.runtimeType == String
            ? User.fromJson(data)
            : User.fromMap(data);
        Storage.saveLoggedUser(user);
        return user;
      }
    } on DioError catch (dioErrorType) {
      if (dioErrorType.response?.statusCode == 401 ||
          dioErrorType.response?.statusCode == 403) {
        if (autoRefreshToken && refreshToken != null) {
          await doRefreshToken(refreshToken);
          return null;
        }
      }
      throw GetUserFailure();
    } on Exception {
      rethrow;
    }
  }

  /// Get a new access token using a refresh token value
  Future<Token> doRefreshToken(String refreshToken) async {
    log('Refreshing token...');
    try {
      var response = await Dio().post(
        '$urlBase/platform/authentication/actions/refreshToken',
        options: Options(
          headers: _headers,
        ),
        data: jsonEncode({'refreshToken': refreshToken}),
      );

      if (response.statusCode == 200) {
        var token = await Storage.getToken();

        final refreshed =
            RefreshTokenModel.fromJson(response.data['jsonToken']);
        var newToken = token.copyWith(
          access_token: refreshed.access_token,
          refresh_token: refreshed.refresh_token,
          expires_in: refreshed.expires_in,
        );

        Storage.saveToken(newToken);

        return newToken;
      }
      throw RefreshTokenFailure();
    } on DioError catch (dioErrorType) {
      if (dioErrorType.response?.statusCode == 401) {
        Storage.clearToken();
        throw RefreshTokenUnauthorized();
      }
      throw RefreshTokenFailure();
    } on Exception {
      rethrow;
    }
  }

  /// Gets the login settings of the tenant associated with the tenantDomain
  ///
  /// Throws a [TenantLogInSettingsFailure] if an exception occurs.
  Future<TenantLoginSettings> getTenantLogInSettings(
      String tenantDomain, String? scope) async {
    try {
      final response = await post(
        Uri.parse(
            '$urlBase/platform/authentication/queries/tenantLoginSettings'),
        headers: _headers,
        body: json.encode({'tenantDomain': tenantDomain, 'scope': scope ?? ''}),
      );

      if (response.statusCode == 400 ||
          response.statusCode == 401 ||
          response.statusCode == 404) throw TenantLogInSettingsIncorrect();

      if (response.statusCode == 200) {
        return TenantLoginSettings.fromJson(response.body);
      }

      throw TenantLogInSettingsFailure();
    } catch (_) {
      rethrow;
    }
  }

  /// Signs in with the provided [UserLogin] infos.
  ///
  /// Throws a [LogInWithEmailAndPasswordFailure] if an exception occurs.
  /// Throws a [LogInWithEmailAndPasswordIncorrect] if user or password
  /// are incorrect.
  Future<void> logInWithEmailAndPassword(UserLogin userLogin) async {
    try {
      var response = await post(
        Uri.parse('$urlBase/platform/authentication/actions/login'),
        headers: _headers,
        body: userLogin.toJson(),
      );

      if (response.statusCode == 401) {
        throw LogInWithEmailAndPasswordIncorrect();
      }

      var token = Token.fromJson(json.decode(response.body)['jsonToken']);

      Storage.saveToken(token);
      initialize();
      _controller.add(AuthenticationStatus.authenticated);
    } on LogInWithEmailAndPasswordIncorrect {
      rethrow;
    } catch (error) {
      throw LogInWithEmailAndPasswordFailure();
    }
  }

  /// Starts the Sign In with SAML Flow.
  ///
  /// Throws a [LogInSamlFailure] if an exception occurs.
  void logInSaml(String tokenStr) {
    try {
      var token = Token.fromJson(tokenStr);
      Storage.saveToken(token);
      initialize();
      _controller.add(AuthenticationStatus.authenticated);
    } catch (error) {
      throw LogInSamlFailure();
    }
  }

  /// Calls the LogOut
  ///
  /// Throws a [LogOutFailure] if an exception occurs.
  /// Throws a [LogOutIncorrect] if token is incorrect.
  Future<void> logOut() async {
    var token = await currentToken;

    if (token.isNotEmpty) {
      try {
        var accessToken =
            AuthenticationUtils.parseTokenValue(token.access_token);
        var response = await post(
          Uri.parse('$urlBase/platform/authentication/actions/logout'),
          headers: {
            'Content-type': 'application/json',
            'Authorization': accessToken
          },
          body: json.encode({'accessToken': token.access_token}),
        );

        if (response.statusCode == 401) throw LogOutIncorrect();

        Storage.clearToken();
        Storage.clearLoggedUser();
        _controller.add(AuthenticationStatus.unauthenticated);
      } catch (error) {
        throw LogOutFailure();
      }
    } else {
      _controller.add(AuthenticationStatus.unauthenticated);
    }
  }

  /// Dispose method is exposed so that the authentication status
  /// StreamController can be closed when it is no longer needed.
  void dispose() => _controller.close();
}
