import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:google_fonts/google_fonts.dart';
import './bloc/password_bloc.dart';
import './bloc/password_form_state.dart';

class PasswordForm extends StatelessWidget {
  PasswordForm({Key? key, required this.username}) : super(key: key);

  final String username;
  final _formKey = GlobalKey<FormState>();

  @override
  Widget build(BuildContext context) {
    return BlocListener<PasswordBloc, PasswordState>(
      listener: (context, state) {
        final formState = state.formState;

        if (formState is PasswordFormIncorrectPassword) {
          ScaffoldMessenger.of(context)
            ..hideCurrentSnackBar()
            ..showSnackBar(const SnackBar(
              content: Text('Verifique seu e-mail e senha e tente novamente'),
            ));
        }

        if (formState is PasswordFormLoadFailure) {
          ScaffoldMessenger.of(context)
            ..hideCurrentSnackBar()
            ..showSnackBar(SnackBar(
              content: Text(formState.exception.toString()),
            ));
        }
      },
      child: Form(
        key: _formKey,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            const SizedBox(height: 48.0),
            _InsertPasswordText(username: username),
            const SizedBox(height: 40.0),
            _PasswordInput(),
            const Spacer(),
            _ContinueButton(_formKey),
            const SizedBox(height: 8),
            _ForgotPasswordButton(),
            const SizedBox(height: 56),
          ],
        ),
      ),
    );
  }
}

class _InsertPasswordText extends StatelessWidget {
  const _InsertPasswordText({required this.username});

  final String username;
  @override
  Widget build(BuildContext context) {
    var name = username.split('@')[0];

    return Wrap(
      direction: Axis.vertical,
      children: [
        Text(
          'Olá,',
          style: GoogleFonts.inter(
              color: Colors.black87, fontSize: 32, fontWeight: FontWeight.w600),
        ),
        Text(
          name,
          style: GoogleFonts.inter(
              color: Colors.black87, fontSize: 32, fontWeight: FontWeight.w600),
        ),
        Text(
          username,
          style: GoogleFonts.inter(
              color: const Color(0xFF808080),
              fontSize: 16,
              fontWeight: FontWeight.w500),
        )
      ],
    );
  }
}

class _PasswordInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<PasswordBloc, PasswordState>(
      builder: (context, state) {
        return TextFormField(
          key: const Key('passwordForm_passwordInput_textField'),
          validator: (value) {
            return state.isValidPassword
                ? null
                : 'Preencha o campo com a sua senha';
          },
          onChanged: (value) => context
              .read<PasswordBloc>()
              .add(PasswordChanged(password: value)),
          keyboardType: TextInputType.text,
          obscureText: !context.read<PasswordBloc>().state.passwordVisible,
          cursorColor: Colors.black87,
          decoration: InputDecoration(
            focusedBorder: const UnderlineInputBorder(
              borderSide: BorderSide(
                color: Colors.black87,
              ),
            ),
            border: const UnderlineInputBorder(),
            prefixIcon: const Icon(
              Icons.lock,
              color: Colors.black87,
            ),
            suffixIcon: IconButton(
              icon: Icon(
                context.read<PasswordBloc>().state.passwordVisible
                    ? Icons.visibility
                    : Icons.visibility_off,
                color: Colors.black87,
              ),
              onPressed: () {
                context.read<PasswordBloc>().add(PasswordVisibilityChange());
              },
            ),
            labelText: 'Senha',
            labelStyle: GoogleFonts.openSans(
                color: Colors.black87,
                fontSize: 16,
                fontWeight: FontWeight.w400),
            helperText: 'Insira a sua senha',
            helperStyle: GoogleFonts.openSans(
                color: const Color(0xFF4D4D4D),
                fontSize: 12,
                fontWeight: FontWeight.w400),
            errorStyle: GoogleFonts.openSans(
                color: const Color(0xFFA94442),
                fontSize: 12,
                fontWeight: FontWeight.w400),
          ),
        );
      },
    );
  }
}

class _ContinueButton extends StatelessWidget {
  const _ContinueButton(this.formKey);

  final GlobalKey<FormState> formKey;

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<PasswordBloc, PasswordState>(
      builder: (context, state) {
        return state.formState is PasswordFormLoadInProgress
            ? const Center(
                child: CircularProgressIndicator(
                backgroundColor: Color(0xFF12A885),
                valueColor: AlwaysStoppedAnimation<Color>(Color(0xFF0E866A)),
              ))
            : ElevatedButton(
                key: const Key('passwordForm_continue_elevatedButton'),
                style: ButtonStyle(
                  shape: MaterialStateProperty.all<RoundedRectangleBorder>(
                      RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(6.0))),
                  backgroundColor: MaterialStateProperty.resolveWith<Color>(
                      (states) => const Color(0xFF0E866A)),
                ),
                onPressed: () {
                  if (formKey.currentState!.validate()) {
                    context.read<PasswordBloc>().add(PasswordSubmitted());
                  }
                },
                child: Container(
                  height: 48,
                  width: double.infinity,
                  alignment: Alignment.center,
                  child: Text(
                    'Entrar',
                    style: GoogleFonts.inter(
                        fontSize: 18.0,
                        fontWeight: FontWeight.w700,
                        color: Colors.white),
                  ),
                ),
              );
      },
    );
  }
}

class _ForgotPasswordButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return TextButton(
      onPressed: () => {
        ScaffoldMessenger.of(context).showSnackBar(const SnackBar(
          content: Text('Ainda não implementado!'),
          duration: Duration(milliseconds: 500),
        ))
      },
      style: ButtonStyle(
        padding: MaterialStateProperty.resolveWith((_) => EdgeInsets.zero),
        overlayColor: MaterialStateColor.resolveWith((_) => Colors.transparent),
      ),
      child: Container(
        height: 48,
        width: double.infinity,
        alignment: Alignment.center,
        child: Text(
          'Recuperar senha',
          style: GoogleFonts.inter(
            fontSize: 18.0,
            fontWeight: FontWeight.w700,
            color: const Color(0xFF008267),
          ),
        ),
      ),
    );
  }
}
