import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:google_fonts/google_fonts.dart';
import '../password/password_screen.dart';
import '../saml/saml_screen.dart';
import './bloc/username_bloc.dart';
import './bloc/username_form_state.dart';

class UsernameForm extends StatelessWidget {
  UsernameForm({Key? key}) : super(key: key);

  final _formKey = GlobalKey<FormState>();

  @override
  Widget build(BuildContext context) {
    return BlocListener<UsernameBloc, UsernameState>(
      listener: (context, state) {
        final formState = state.formState;

        if (formState is UsernameFormLoadFailure) {
          ScaffoldMessenger.of(context)
            ..hideCurrentSnackBar()
            ..showSnackBar(SnackBar(
              content: Text(formState.exception.toString()),
            ));
        }

        if (formState is UsernameFormLoadSuccess) {
          if (formState.tenantLoginSettings.loginProviders.saml != null) {
            Navigator.push(
              context,
              MaterialPageRoute(
                builder: (_) => SamlScreen(
                  username: formState.username,
                ),
              ),
            );
            return;
          }

          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (_) => PasswordScreen(
                username: formState.username,
              ),
            ),
          );
        }
      },
      child: Form(
        key: _formKey,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            const SizedBox(height: 48.0),
            _InsertEmailText(),
            const SizedBox(height: 40.0),
            _UsernameInput(),
            const Spacer(),
            _ContinueButton(_formKey),
            const SizedBox(height: 56),
          ],
        ),
      ),
    );
  }
}

class _InsertEmailText extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Wrap(
      direction: Axis.vertical,
      children: [
        Text(
          'Insira',
          style: GoogleFonts.inter(
              color: Colors.black87, fontSize: 32, fontWeight: FontWeight.w600),
        ),
        Text(
          'seu e-mail',
          style: GoogleFonts.inter(
              color: Colors.black87, fontSize: 32, fontWeight: FontWeight.w600),
        ),
      ],
    );
  }
}

class _UsernameInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return BlocBuilder<UsernameBloc, UsernameState>(
      builder: (context, state) {
        return TextFormField(
          key: const Key('usernameForm_usernameInput_textField'),
          validator: (value) =>
              state.isValidUsername ? null : 'E-mail inválido',
          onChanged: (value) => context
              .read<UsernameBloc>()
              .add(UsernameChanged(username: value)),
          keyboardType: TextInputType.emailAddress,
          cursorColor: Colors.black87,
          decoration: InputDecoration(
            focusedBorder: const UnderlineInputBorder(
              borderSide: BorderSide(
                color: Colors.black87,
              ),
            ),
            border: const UnderlineInputBorder(),
            prefixIcon: const Icon(
              Icons.email,
              color: Colors.black87,
            ),
            labelText: 'E-mail',
            labelStyle: GoogleFonts.openSans(
                color: Colors.black87,
                fontSize: 16,
                fontWeight: FontWeight.w400),
            helperText: 'Insira o e-mail que você utiliza na Senior X',
            helperStyle: GoogleFonts.openSans(
                color: const Color(0xFF4D4D4D),
                fontSize: 12,
                fontWeight: FontWeight.w400),
            errorStyle: GoogleFonts.openSans(
                color: const Color(0xFFA94442),
                fontSize: 12,
                fontWeight: FontWeight.w400),
          ),
        );
      },
    );
  }
}

class _ContinueButton extends StatelessWidget {
  const _ContinueButton(this.formKey);

  final GlobalKey<FormState> formKey;

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<UsernameBloc, UsernameState>(
      builder: (context, state) {
        return state.formState is UsernameFormLoadInProgress
            ? const Center(
                child: CircularProgressIndicator(
                backgroundColor: Color(0xFF12A885),
                valueColor: AlwaysStoppedAnimation<Color>(Color(0xFF0E866A)),
              ))
            : ElevatedButton(
                key: const Key('usernameForm_continue_elevatedButton'),
                style: ButtonStyle(
                  shape: MaterialStateProperty.all<RoundedRectangleBorder>(
                      RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(6.0))),
                  backgroundColor: MaterialStateProperty.resolveWith<Color>(
                      (states) => const Color(0xFF0E866A)),
                ),
                onPressed: () {
                  if (formKey.currentState!.validate()) {
                    context.read<UsernameBloc>().add(UsernameSubmitted());
                  }
                },
                child: Container(
                  height: 48,
                  width: double.infinity,
                  alignment: Alignment.center,
                  child: Text(
                    'Próximo',
                    style: GoogleFonts.inter(
                        fontSize: 18.0,
                        fontWeight: FontWeight.w700,
                        color: Colors.white),
                  ),
                ),
              );
      },
    );
  }
}
